@@ -9,14 +9,14 @@
 #' More formally, \code{penhdfeppml_int} performs iteratively re-weighted least squares (IRLS) on a
 #' transformed model, as described in Breinlich, Corradi, Rocha, Ruta, Santos Silva and Zylkin (2020).
 #' In each iteration, the function calculates the transformed dependent variable, partials out the fixed
-#' effects (calling \code{collapse::fhdwithin}) and then and then calls \code{glmnet::glmnet} if the selected
+#' effects (calling \code{collapse::fhdwithin}) and then and then calls \code{glmnet} if the selected
 #' penalty is lasso (the default). If the user selects ridge, the analytical solution is instead
 #' computed directly using fast C++ implementation.
 #'
 #' For information on the plugin lasso method, see \link{penhdfeppml_cluster_int}.
 #'
 #' @param lambda Penalty parameter (a number).
-#' @param glmnettol Tolerance parameter to be passed on to \code{glmnet::glmnet}.
+#' @param glmnettol Tolerance parameter to be passed on to \code{glmnet}.
 #' @param penalty A string indicating the penalty type. Currently supported: "lasso" and "ridge".
 #' @param penweights Optional: a vector of coefficient-specific penalties to use in plugin lasso when
 #'     \code{method == "plugin"}.
@@ -26,6 +26,11 @@
 #'     coefficient-specific penalty weights (see details). Otherwise, a single global penalty is used.
 #' @param debug Logical. If \code{TRUE}, this helps with debugging penalty weights by printing output
 #'    of the first iteration to the console and stopping the estimation algorithm.
+#' @param colcheck_x Logical. If \code{TRUE}, this checks collinearity between the independent variables and drops the
+#' collinear variables.
+#' @param colcheck_x_fes Logical. If \code{TRUE}, this checks whether the independent variables are perfectly explained
+#' by the fixed effects drops those that are perfectly explained.
+#' @param gamma_val Numerical value that determines the regularization threshold as defined in Belloni, Chernozhukov, Hansen, and Kozbur (2016). NULL default sets parameter to 0.1/log(n).
 #' @inheritParams hdfeppml_int
 #'
 #' @return If \code{method == "lasso"} (the default), an object of class \code{elnet} with the elements
@@ -66,19 +71,26 @@
 #' \donttest{reg <- penhdfeppml_int(y = y, x = x, fes = fes, lambda = 0.1, penalty = "ridge")}
 #'
 #' @inheritSection hdfeppml_int References
+#' @importFrom stats gaussian
+#' @importFrom stats var
+#' @importFrom utils head
+#' @importFrom devtools load_all
 
 penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmnettol = 1e-12,
-                        penalty = "lasso", penweights = NULL, saveX = TRUE, mu = NULL, colcheck = TRUE,
-                        init_z = NULL, post = FALSE, verbose = FALSE, standardize = TRUE,
-                        method = "placeholder", cluster = NULL, debug = FALSE) {
-
+                            penalty = "lasso", penweights = NULL, saveX = TRUE, mu = NULL, colcheck_x = FALSE, colcheck_x_fes = TRUE,
+                            init_z = NULL, post = FALSE, verbose = FALSE, standardize = TRUE,
+                            method = "placeholder", cluster = NULL, debug = FALSE, gamma_val=NULL) {
+  xnames <- colnames(x)
+  old_x <- x
+  old_y <- y
+  old_fes <- fes
   # implements plugin method; calls penhdfeppml_cluster_int subcommand
   if (method == "plugin") {
     penreg <- penhdfeppml_cluster_int(y = y, x = x, fes = fes, cluster = cluster, tol = tol,
-                                  hdfetol = hdfetol, glmnettol = glmnettol, penalty = penalty,
-                                  penweights = penweights, saveX = saveX,
-                                  mu = mu, colcheck = colcheck, K = 15, init_z = init_z, post = FALSE,
-                                  verbose = verbose, lambda = NULL)
+                                      hdfetol = hdfetol, glmnettol = glmnettol, penalty = penalty,
+                                      penweights = penweights, saveX = saveX,
+                                      mu = mu, colcheck_x = colcheck_x, colcheck_x_fes = colcheck_x_fes, K = 15, init_z = init_z, post = FALSE,
+                                      verbose = verbose, lambda = NULL, gamma_val=gamma_val)
   }
   else {
 
@@ -87,9 +99,9 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
     rownames(b) <- colnames(x)
     include_x <- 1:ncol(x)
 
-    if (is.null(penweights)) {
-      penweights <- rep(1, length(include_x))   #note this is the default used by glmnet. Using "NULL" actually gives you incorrect weights.
-    }
+   # if (is.null(penweights)) {
+   #    penweights <- rep(1, length(include_x))   #note this is the default used by glmnet. Using "NULL" actually gives you incorrect weights.
+   #  } #N: moved this below colchecks, because include_x gets updated
 
     if (length(penweights) > length(include_x)){
       print("penweights needs to be same length as number of x variables")
@@ -97,17 +109,41 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
     }
 
     # collinearity check
-    if (colcheck == TRUE) {
-      include_x <- collinearity_check(y, x, fes, 1e-6)
-      x <- x[, include_x]
-      if (!is.null(penweights)) {
-        penweights = penweights[include_x]
-      }
+    if (colcheck_x==TRUE & colcheck_x_fes==TRUE){
+      include_x <- collinearity_check(y,x,fes,1e-6, colcheck_x=colcheck_x, colcheck_x_fes=colcheck_x_fes)
+      x <- x[,include_x]
+      colnames(x) <- xnames[include_x]
+      xnames <- xnames[include_x]
+      colcheck_x_post = FALSE
+      colcheck_x_fes_post = FALSE
+    }
+    if (colcheck_x==FALSE & colcheck_x_fes==FALSE){
+      colcheck_x_post = TRUE
+      colcheck_x_fes_post = TRUE
+    }
+    if (colcheck_x==TRUE & colcheck_x_fes==FALSE){
+      include_x <- collinearity_check(y,x,fes,1e-6, colcheck_x=colcheck_x, colcheck_x_fes=colcheck_x_fes)
+      x <- x[,include_x]
+      colnames(x) <- xnames[include_x]
+      xnames <- xnames[include_x]
+      colcheck_x_post = TRUE
+      colcheck_x_fes_post = FALSE
+    }
+    if (colcheck_x==FALSE & colcheck_x_fes == TRUE){
+      include_x <- collinearity_check(y,x,fes,1e-6, colcheck_x=colcheck_x, colcheck_x_fes=colcheck_x_fes)
+      x <- x[,include_x]
+      colnames(x) <- xnames[include_x]
+      xnames <- xnames[include_x]
+      colcheck_x_post = FALSE
+      colcheck_x_fes_post = TRUE
+    }
+
+    if (is.null(penweights)) {
+      penweights <- rep(1, length(include_x))   #note this is the default used by glmnet. Using "NULL" actually gives you incorrect weights.
     }
 
     # number of obs (needed for deviance)
     n <- length(y)
-
     # estimation algorithm (this implements a version of the algorithm from p. 110 of CGZ SJ 2020 but with penalized WLS in place of the WLS step)
     crit <- 1
     old_deviance <-0
@@ -115,11 +151,19 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
     while (crit > tol) {
       iter <- iter + 1
 
-      if (iter == 1) {
+      if(iter > 50){message("Lasso exceeded 50 iterations. Break loop and return last model."); break}
 
+      if (iter == 1) {
         # initilize "mu"
-        if (is.null(mu)) mu  <- (y + mean(y))/2
+        if (is.null(mu)){
+          only_fes <- hdfeppml_int(y, fes=fes, tol = 1e-8, hdfetol = 1e-4, colcheck_x = FALSE, colcheck_x_fes = FALSE, mu = NULL, saveX = TRUE,
+                                   init_z = NULL, verbose = FALSE, maxiter = 1000, cluster = NULL, vcv = TRUE)
+          mu <- only_fes$mu
+          #      mu <- mu[mu>0]
+          # print(length(mu))
+        }
         z   <- (y - mu)/mu + log(mu)
+        #     z <- z[mu>0]
         eta <- log(mu)
         last_z <- z
         if (is.null(init_z)) {
@@ -127,28 +171,96 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
         } else {
           reg_z <- init_z
         }
-        reg_x  <- x
+        reg_x  <- x[which(mu>0),]
+        n <- length(z)
 
       } else {
         last_z <- z
+        n <- length(mu)
+        # print(n)
+        #        y <- y[mu>0]
         z <- (y - mu)/mu + log(mu)
-        reg_z  <- matrix(z - last_z + z_resid)
-        reg_x  <- x_resid
+        #z[which(mu==1e-190)] <- 1e-190
+        #z[which(z==Inf)] <- 1
+        reg_z  <- matrix(z - last_z[which(mu>0)] + z_resid[which(mu>0)])
+        #print("reg_z")
+        #print(sum(is.na(reg_z)))
+        reg_x  <- x_resid[which(mu>0),]
         ## colnames(reg_x)   <- colnames(x)
       }
-
+      #      fes <- lapply(fes, "[", which(mu>0))
+      # print(length(reg_z))
+      # print(dim(reg_x)[1])
+      # print(length(mu))
+      # print(length(fes[[1]]))
+      # print(length(fes[[2]]))
+      # print(length(fes[[3]]))
       # HDFE estimation works with the residuals of z and x after purging them of the FEs (see CGZ Stata Journal 2020)
-      z_resid <- collapse::fhdwithin(reg_z, fes, w = mu)
-      x_resid <- collapse::fhdwithin(reg_x, fes, w = mu)
+      if(!missing(fes)){
+        #   print("fes not missing")
+        if(is.null(fes)){
+          z_resid <- collapse::fwithin(x=reg_z, g=factor(rep(1,length(reg_z))), w = mu)
+          if(!missing(x)){
+            x_resid <- collapse::fwithin(x=reg_x,g=factor(rep(1,length(reg_z))), w = mu)
+          }
+        }else{
+          # print(length(reg_z))
+          # print(length(fes[[1]]))
+          # print(length(mu))
+          z_resid <-  collapse::fhdwithin(reg_z, fes, w = mu)
+          if(!missing(x)){
+            x_resid <- collapse::fhdwithin(reg_x, fes, w = mu)
+          }
+          # print("postfhd1")
+        }
+      } else {
+        print("fes missing")
+        z_resid <- reg_z
+        if(!missing(x)){
+          x_resid <- reg_x
+        }
+      }
+
+      # if(sd(z_resid)==Inf){
+      #   print("z_resid")
+      #   print(head(z_resid))
+      #   print(sum(is.na(z_resid)))
+      #   print(mean(z_resid))
+      #   print(sd(z_resid))
+      # }
+      # if(length(reg_z)!=length(z_resid)){
+      #   mu_t <- mu
+      #   reg_z_t <- reg_z
+      #   temp1 <- (collapse::fhdwithin(reg_z, fes, w = mu, eps = hdfetol))
+      #   temp2 <- (lfe::demeanlist(reg_z, fes, weights = sqrt(mu), eps = hdfetol))
+      #   print("check")
+      #   print(all.equal(temp1,temp2))
+      #   print(sum(is.na(temp2)))
+      # }
+
+      #       print(length(z))
+      #       print(length(reg_z))
+      #print("mu")
+      #print(length(mu))
+      #print(reg_z)
+      # print(sum(mu==0))
+      # print("data")
+      #  print(length(z_resid))
+      #  print(dim(reg_x))
+      # print(length(z_resid))
+      # print(dim(x_resid))
 
       if (is.null(penweights)) {
-        #penreg <- glmnet::glmnet(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = lambda, thresh = glmnettol, standardize = standardize)
+        print("null penweights")
+        #penreg <- glmnet(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = lambda, thresh = glmnettol, standardize = standardize, family=gaussian(link = "identity"), warm.g=NULL)
       }
       else{
         if(debug) {
+          print("sum penweights")
           print(sum(penweights))
         }
         if(debug) {
+          print("penweights")
           print(penweights)
         }
         penweights = penweights * length(include_x) / sum(penweights)
@@ -157,13 +269,13 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
         }
       }
 
-# The SCAD option is DISABLED:
-#      if (penalty == "SCAD") {  ## !! currently *very* slow... can be sped up using warm starts??
-#        wz_resid <- sqrt(mu) * z_resid
-#        wx_resid <- sqrt(mu) * x_resid
-#        penreg <- ncvreg::ncvreg(wx_resid, wz_resid, penalty = "SCAD", lambda = lambda)  # add penalty weights
-#
-#      }  else if (penalty == "ridge") {
+      # The SCAD option is DISABLED:
+      #      if (penalty == "SCAD") {  ## !! currently *very* slow... can be sped up using warm starts??
+      #        wz_resid <- sqrt(mu) * z_resid
+      #        wx_resid <- sqrt(mu) * x_resid
+      #        penreg <- ncvreg::ncvreg(wx_resid, wz_resid, penalty = "SCAD", lambda = lambda)  # add penalty weights
+      #
+      #      }  else if (penalty == "ridge") {
       if (penalty == "ridge") {
         penreg <- fastridge(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = n * lambda,
                             standardize = standardize) # ,penalty.factor=penweights
@@ -175,18 +287,18 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
         }
         if (debug) {
           penreg <- glmnet::glmnet(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = lambda,
-                                   thresh = glmnettol, standardize = standardize)
+                                thresh = glmnettol, standardize = standardize, family=gaussian(link = "identity"))
           print((penreg$beta))
           print(penweights)
         }
         if (debug) {
           penreg <- glmnet::glmnet(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = lambda,
-                                   thresh = glmnettol, standardize = standardize)
+                                thresh = glmnettol, standardize = standardize, family=gaussian(link = "identity"))
           print((penreg$beta))
           print(penweights)
         }
         penreg <- glmnet::glmnet(x = x_resid, y = z_resid, weights = mu/sum(mu), lambda = lambda,
-                                 thresh = glmnettol, penalty.factor = penweights, standardize = standardize)
+                              thresh = glmnettol, penalty.factor = penweights, standardize = standardize, family=gaussian(link = "identity"))
         if (debug) {
           print((penreg$beta))
           stop()
@@ -198,13 +310,29 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
       residuals <- z_resid - x_resid %*% b[include_x]
 
       mu <- as.numeric(exp(z - residuals))
+      #print("smaller zero")
+      #     print(length(which(mu <= 0)))
+      #      mu <- mu[which(mu > 0)]
+      mu[which(mu < 1e-190)] <- 1e-190
+      mu[mu > 1e190] <- 1e190
+      #y <- y[which(mu > 0)]
+      # print("mu")
+      # print(length(mu[which(mu == 1e-190)]))
+      # print(length(mu[which(mu == 1e16)]))
 
       # calculate deviance
-      temp <-  -(y * log(y/mu) - (y-mu))
+      temp <-  -(y * log(y/mu) - (y-mu)) # Problem sits here: - Inf + Inf = NaN
       temp[which(y == 0)] <- -mu[which(y == 0)]
+      #temp[which(mu==Inf)] <- 0
+      # print("temp")
+      # print(y[which(is.na(temp))])
+      # print(mu[which(is.na(temp))])
 
       deviance <- -2 * sum(temp)/n
-
+      # print("pen")
+      # print(temp[which(is.na(temp))])
+      # print(mu[which(is.na(temp))])
+      # print(y[which(is.na(temp))])
       if(deviance<0) deviance = 0
 
       delta_deviance <- old_deviance - deviance
@@ -216,15 +344,12 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
       denom_crit = max(c( min(c(deviance, old_deviance)) , 0.1 ))
       crit = abs(delta_deviance) / denom_crit
 
-      #print(deviance)
       if (verbose == TRUE) {
-        #print(deviance)
         print(crit)
       }
 
       old_deviance <- deviance
     }
-
     ## elements to return
     k   <- ncol(matrix(x))
     n   <- length(y)
@@ -241,10 +366,9 @@ penhdfeppml_int <- function(y, x, fes, lambda, tol = 1e-8, hdfetol = 1e-4, glmne
       x_select <- x_resid[, as.numeric(penreg$beta) != 0]
       if (length(x_select) != 0){
         ppml_temp <- hdfeppml_int(y = y, x = x_select, fes = fes, tol = tol, hdfetol = hdfetol,
-                              mu = penreg$mu, colcheck = FALSE)
-
+                                  mu = penreg$mu, colcheck_x = colcheck_x_post, colcheck_x_fes = colcheck_x_fes_post)
         penreg$pencoefs <- penreg$beta
-        penreg$beta[which(penreg$beta != 0), 1]  <- ppml_temp$coefficients
+        penreg$beta[which(as.logical(penreg$beta != 0)), 1]  <- ppml_temp$coefficients
         b[include_x] <- penreg$beta
         mu    <- ppml_temp$mu
         bic   <- ppml_temp$bic
